

## ------------------------------------------------------------------------ ##
##  R scripts for " A New Parametric Approach to Smoothing and Forecasting  ##
##                 of Mortality "                                           ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts for reference tables  -  Script n•01              ##
## ------------------------------------------------------------------------ ##
##  Description   Script to load, create and save reference tables. We      ##
##                consider tables from INSEE and TGHF05.                    ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 19/10/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...")

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("Misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ------------------------------------------------------------------------ ##
##           	                   Remarks                                  ##
## ------------------------------------------------------------------------ ##

## - Focus on behaviour after 30 years old to 90
## - Period taken in account: 1996:2035

## ------------------------------------------------------------------------ ##
##           	                    TGHF05                                  ##
## ------------------------------------------------------------------------ ##

## ---------------- Load tables

TG05_f <- read.table(".../TGF05.csv",header=T,sep=",", colClasses = "character")
TG05_f <- TG05_f[2:122,2:ncol(TG05_f)]
TG05_h <- read.table(".../TGH05.csv",header=T,sep=",", colClasses = "character")
TG05_h <- TG05_h[2:122,2:ncol(TG05_h)]

TG05_L_h <- matrix(as.numeric(as.matrix(TG05_h)), nrow(TG05_h), ncol(TG05_h))
TG05_L_f <- matrix(as.numeric(as.matrix(TG05_f)), nrow(TG05_f), ncol(TG05_f))

## --- Surface plot

x11();
surface_plot(TG05_L_h, expression(paste(TGH[05])), c(0,120,1900,2005))
x11();
surface_plot(TG05_L_f, expression(paste(TGF[05])), c(0,120,1900,2005))

## ---------------- Compute number of death by generation

TG05_D_h <- TG05_L_h[1:120,] - TG05_L_h[-1,]
TG05_D_f <- TG05_L_f[1:120,] - TG05_L_f[-1,]

## ---------------- Compute forces of mortality by generation

TG05_m_h_gen  <- TG05_D_h / ( - TG05_L_h[1:120,] * (TG05_D_h/TG05_L_h[1:120,] ) / log(1 - (TG05_D_h /TG05_L_h[1:120,] )))
TG05_m_f_gen  <- TG05_D_f / ( - TG05_L_f[1:120,] * (TG05_D_f/TG05_L_f[1:120,] ) / log(1 - (TG05_D_f /TG05_L_f[1:120,] )))

## ---------------- Re-arrange tables mu(x,t) = mu_gen(x,t-x)
			
TG05_m_h <- TG05_m_f <- matrix(,nrow(TG05_m_h_gen),136)
	for (t in 1:136) {
		for (x in 0:(nrow(TG05_m_h_gen)-1)){
			y <- t - x 
			if (y > 0 && y <= 106) {
			TG05_m_h[x+1, t] <- TG05_m_h_gen[x+1,  y]
			TG05_m_f[x+1, t] <- TG05_m_f_gen[x+1,  y] } } }

## ---------------- Keep only years 1997:2019 (years forwhich we have sufficient data)

colnames(TG05_m_f) <- colnames(TG05_m_h) <- 1900:2035

TG05_m_h <- TG05_m_h[31:91, as.character(1996:2035)]		
TG05_m_f <- TG05_m_f[31:91, as.character(1996:2035)]

## --- Surface plot

## - Forces of mortality

x11();
surface_plot(TG05_m_h, expression(paste("TG05 m"[h])), c(30,90,1996,2035))
x11();
surface_plot(TG05_m_f, expression(paste("TG05 m"[f])), c(30,90,1996,2035))

## - Forces of mortality (log scale)

x11();
surface_plot(log(TG05_m_h), expression(paste("TG05 ",log(m[h]))), c(30,90,1996,2035))
x11();
surface_plot(log(TG05_m_f), expression(paste("TG05 ",log(m[f]))), c(30,90,1996,2035))

## ---------------- Save tables

save(TG05_m_h, file = "TG05_m_h.RData")
save(TG05_m_f, file = "TG05_m_f.RData")

## ------------------------------------------------------------------------ ##
##           	                    INSEE                                   ##
## ------------------------------------------------------------------------ ##

## --- We have at our disposal 2 tables: 
## --- Table A 1977:2010
## --- Table B 2007:2060

## ---------------- Table A

INSEE_A_L_h <- read.table(".../INSEE_A_h.csv",header=T,sep=",", colClasses = "numeric")
INSEE_A_L_f <- read.table(".../INSEE_A_f.csv",header=T,sep=",", colClasses = "numeric")
INSEE_A_L_h <- as.matrix(INSEE_A_L_h); INSEE_A_L_f <- as.matrix(INSEE_A_L_f)
colnames(INSEE_A_L_h) <- colnames(INSEE_A_L_f) <- 1977:2010

## --- Surface plot

x11();
surface_plot(INSEE_A_L_h, expression(paste(INSEE_A[h])), c(0,99,1977,2010))
x11();
surface_plot(INSEE_A_L_f, expression(paste(INSEE_A[f])), c(0,99,1977,2010))

## --- Compute number of death

INSEE_A_D_h <- INSEE_A_L_h[1:99,] - INSEE_A_L_h[-1,]
INSEE_A_D_f <- INSEE_A_L_f[1:99,] - INSEE_A_L_f[-1,]

## --- Compute forces of mortality

INSEE_A_ETR_h <-  - INSEE_A_L_h[1:99,] * (INSEE_A_D_h/INSEE_A_L_h[1:99,] ) / log(1 - (INSEE_A_D_h /INSEE_A_L_h[1:99,] ))
INSEE_A_ETR_f <-  - INSEE_A_L_f[1:99,] * (INSEE_A_D_f/INSEE_A_L_f[1:99,] ) / log(1 - (INSEE_A_D_f /INSEE_A_L_f[1:99,] ))

INSEE_A_m_h  <- INSEE_A_D_h / INSEE_A_ETR_h
INSEE_A_m_f  <- INSEE_A_D_f / INSEE_A_ETR_f

## --- Surface plot

## - Forces of mortality

x11();
surface_plot(INSEE_A_m_h, expression(paste("INSEE_A m"[h])), c(0,98,1977,2010))
x11();
surface_plot(INSEE_A_m_f, expression(paste("INSEE_A m"[f])), c(0,98,1977,2010))

## - Forces of mortality (log scale)

x11();
surface_plot(log(INSEE_A_m_h), expression(paste("INSEE_A ",log(m[h]))), c(0,98,1977,2010))
x11();
surface_plot(log(INSEE_A_m_f), expression(paste("INSEE_A ",log(m[f]))), c(0,98,1977,2010))

## ---------------- Table B

## --- Population

INSEE_B_L_h <- read.table(".../INSEE_B_L_h.csv",header=T,sep=",", colClasses = "numeric", skip=4, nrow=108)
INSEE_B_L_f <- read.table(".../INSEE_B_L_f.csv",header=T,sep=",", colClasses = "numeric", skip=4, nrow=108)
INSEE_B_L_h <- as.matrix(INSEE_B_L_h[,2:ncol(INSEE_B_L_h)])
INSEE_B_L_f <- as.matrix(INSEE_B_L_f[,2:ncol(INSEE_B_L_f)])

## --- Number of death

INSEE_B_D_h <- read.table(".../INSEE_B_D_h.csv",header=T,sep=",", colClasses = "character", skip=4, nrow=108, dec=".")
INSEE_B_D_f <- read.table(".../INSEE_B_D_f.csv",header=T,sep=",", colClasses = "character", skip=4, nrow=108, dec=".")
INSEE_B_D_h <- matrix(as.numeric(sub(",",".",as.matrix(INSEE_B_D_h[,2:ncol(INSEE_B_D_h)]))), nrow(INSEE_B_D_h), ncol(INSEE_B_D_h)-1)
INSEE_B_D_f <- matrix(as.numeric(sub(",",".",as.matrix(INSEE_B_D_f[,2:ncol(INSEE_B_D_f)]))), nrow(INSEE_B_D_f), ncol(INSEE_B_D_f)-1)
colnames(INSEE_B_L_h) <- colnames(INSEE_B_L_f) <- colnames(INSEE_B_D_h) <- colnames(INSEE_B_D_f) <- 2007:2060

## --- Surface plot

x11();
surface_plot(INSEE_B_L_h, expression(paste(INSEE_B_L[h])), c(0,107,2007,2060))
x11();
surface_plot(INSEE_B_L_f, expression(paste(INSEE_B_L[f])), c(0,107,2007,2060))

x11();
surface_plot(INSEE_B_D_h, expression(paste(INSEE_B_D[h])), c(0,107,2007,2060))
x11();
surface_plot(INSEE_B_D_f, expression(paste(INSEE_B_D[f])), c(0,107,2007,2060))

## --- Compute forces of mortality

INSEE_B_ETR_h <- - INSEE_B_L_h * (INSEE_B_D_h/INSEE_B_L_h) / log(1 - (INSEE_B_D_h /INSEE_B_L_h ))
INSEE_B_ETR_f <- (- INSEE_B_L_f * (INSEE_B_D_f/INSEE_B_L_f) / log(1 - (INSEE_B_D_f /INSEE_B_L_f )))

INSEE_B_m_h  <- (INSEE_B_D_h / INSEE_B_ETR_h)[1:99,]
INSEE_B_m_f  <- (INSEE_B_D_f / INSEE_B_ETR_f)[1:99,]

## --- Surface plot

## - Forces of mortality

x11();
surface_plot(INSEE_B_m_h, expression(paste("INSEE_B m"[h])),c(0,98,2007,2060))
x11();
surface_plot(INSEE_B_m_f, expression(paste("INSEE_B m"[f])),c(0,98,2007,2060))

## - Forces of mortality (log scale)

x11();
surface_plot(log(INSEE_B_m_h), expression(paste("INSEE_B ",log(m[h]))),c(0,98,2007,2060))
x11();
surface_plot(log(INSEE_B_m_f), expression(paste("INSEE_B ",log(m[f]))),c(0,98,2007,2060))

## ---------------- Check the correspondance for the years 2007:2010

x11()
plot(0:98, log(INSEE_A_m_h[,"2007"]), type="l")
points(0:98, log(INSEE_B_m_h[,"2007"]), type="l", col=2)

x11();
plot(0:98, log(INSEE_A_m_h[,"2008"]), type="l")
points(0:98, log(INSEE_B_m_h[,"2008"]), type="l", col=2)

x11();
plot(0:98, log(INSEE_A_m_h[,"2009"]), type="l")
points(0:98, log(INSEE_B_m_h[,"2009"]), type="l", col=2)

x11();
plot(0:98, log(INSEE_A_m_h[,"2010"]), type="l")
points(0:98, log(INSEE_B_m_h[,"2010"]), type="l", col=2)

## ---------------- Keep Age 30:90 and Table A for years 2007:2010
## ---------------- Merge the tables

INSEE_m_h <- cbind(INSEE_A_m_h[31:91, as.character(1996:2010)], INSEE_B_m_h[31:91, as.character(2011:2060)])
INSEE_m_f <- cbind(INSEE_A_m_f[31:91, as.character(1996:2010)], INSEE_B_m_f[31:91, as.character(2011:2060)])

## --- Surface plot

## - Forces of mortality

x11()
surface_plot(INSEE_m_h, expression(paste("INSEE m"[h])),c(30,90,1996,2060))
x11()
surface_plot(INSEE_m_f, expression(paste("INSEE m"[f])),c(30,90,1996,2060))

## - Forces of mortality (log scale)

x11()
surface_plot(log(INSEE_m_h), expression(paste("INSEE_B ",log(m[h]))),c(30,90,1997,2060))
x11()
surface_plot(log(INSEE_m_f), expression(paste("INSEE_B ",log(m[f]))),c(30,90,1997,2060))

## --- Remark: We could maybe smooth the tables to obtain a smooth transition between the years 2010 and 2011, and smooth the peaks for older ages...

## --- Smooth by local polynomial:

require(locfit)

fam <- "poisson"; lk <- "log"; k <- "gaussian"

Age <- 30:90; Year <- 1996:2035
new.data <- expand.grid(age=Age, year=Year)

## - Male population

y <- cbind(INSEE_A_D_h[Age+1, as.character(1996:2010)], INSEE_B_D_h[Age+1, as.character(2011:2035)])
x1 <- rep(Age,length(Year))
x2 <- rep(Year,each= length(Age))
x3 <- cbind(INSEE_A_ETR_h[Age+1, as.character(1996:2010)], INSEE_B_ETR_h[Age+1, as.character(2011:2035)])

nber.p <- get.enp.target(17); nber.p
span <- nber.p / (length(Age)*length(Year)); span

## - Fit the model

mod_h <- locfit(c(y) ~ lp(c(x1),c(x2),deg = 2, nn = span,  scale=1),ev=dat(), family = fam, link = lk, kern = k, weights=c(x3))

fit_h <- matrix(fitted(mod_h,new.data),length(Age),length(Year))
colnames(fit_h) <- as.character(Year)

x11();
surface_plot(log(fit_h), expression(paste("log ", widehat(mu)[h])), c(min(Age),max(Age), min(Year), max(Year)))

INSEE_m_h <- fit_h

## - Female population

y <- cbind(INSEE_A_D_f[Age+1, as.character(1996:2010)], INSEE_B_D_f[Age+1, as.character(2011:2035)])
x1 <- rep(Age,length(Year))
x2 <- rep(Year,each= length(Age))
x3 <- cbind(INSEE_A_ETR_f[Age+1, as.character(1996:2010)], INSEE_B_ETR_f[Age+1, as.character(2011:2035)])

nber.p <- get.enp.target(17); nber.p
span <- nber.p / (length(Age)*length(Year)); span

## - Fit the model

mod_f <- locfit(c(y) ~ lp(c(x1),c(x2),deg = 2, nn = span,  scale=1),ev=dat(), family = fam, link = lk, kern = k, weights=c(x3))

fit_f <- matrix(fitted(mod_f,new.data),length(Age),length(Year))
colnames(fit_f) <- as.character(Year)

x11();
surface_plot(log(fit_f), expression(paste("log ", widehat(mu)[f])), c(min(Age),max(Age), min(Year), max(Year)))

INSEE_m_f <- fit_f

## ---------------- Save tables

save(INSEE_m_h, file = "INSEE_m_h.RData")
save(INSEE_m_f, file = "INSEE_m_f.RData")


