

## ------------------------------------------------------------------------ ##
##  R scripts for " A New Parametric Approach to Smoothing and Forecasting  ##
##                 of Mortality "                                           ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts to smooth relational model  -  Script n•03.2      ##
## ------------------------------------------------------------------------ ##
##  Description   Script to smooth the forces of mortality by relational    ##
##                models.                                                   ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 19/10/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...")

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("Misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ------------------------------------------------------------------------ ##
##           	                Load the data                               ##
## ------------------------------------------------------------------------ ##

load("D_h.RData")

load("ETR_h.RData")

load("TG05_m_h.RData")

load("INSEE_m_h.RData")

## ------------------------------------------------------------------------ ##
##           	                 The models                                 ##
## ------------------------------------------------------------------------ ##

## --- Relational model

## - Mod.5.INSEE: regression on logit of mu_INSEE_x
## - Mod.5.TG05: regression on logit of mu_TGHF05_x

Age <- 30:90; Year <- 1996:2007

## ------------------------------------------------------------------------ ##
##           	                   mod.5.INSEE                              ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

y <- D_h
x1 <- ETR_h
x2 <- INSEE_m_h[, as.character(Year)]

## --- Compute the fit

mod.m5.INSEE_h <- rel.mod(y/x1, x2, x1)
fit.m5.INSEE_h <- mod.m5.INSEE_h$x_smooth

x11();
surface_plot(fit.m5.INSEE_h, expression(widehat(mu)[h]), c(min(Age),max(Age), min(Year), max(Year)))

x11();
surface_plot(log(fit.m5.INSEE_h), expression(log(widehat(mu)[h])), c(min(Age),max(Age), min(Year), max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.5.TG05                               ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

y <- D_h
x1 <- ETR_h
x2 <- TG05_m_h[, as.character(Year)]

## --- Compute the fit

mod.m5.TG05_h <- rel.mod(y/x1, x2, x1)
fit.m5.TG05_h <- mod.m5.TG05_h$x_smooth

x11();
surface_plot(fit.m5.TG05_h, expression(widehat(mu)[h]), c(min(Age),max(Age), min(Year), max(Year)))

x11();
surface_plot(log(fit.m5.TG05_h), expression(log(widehat(mu)[h])), c(min(Age),max(Age), min(Year), max(Year)))

## ------------------------------------------------------------------------ ##
##           	       Estimated regression parameters                      ##
## ------------------------------------------------------------------------ ##

mod.m5.INSEE_h$rslt;  mod.m5.TG05_h$rslt

## ------------------------------------------------------------------------ ##
##           	               Save the models                              ##
## ------------------------------------------------------------------------ ##

mod.rel_h <- vector("list",2)
mod.rel_h[[1]] <- mod.m5.INSEE_h
mod.rel_h[[2]] <- mod.m5.TG05_h

names(mod.rel_h) <- names(mod.rel_f) <- c("M5.INSEE", "M5.TG05")

save(mod.rel_h, file="mod.rel_h.RData")

## ------------------------------------------------------------------------ ##
##           	                 Save the fits                              ##
## ------------------------------------------------------------------------ ##

fit.rel_h <- vector("list",2)
fit.rel_h[[1]] <- fit.m5.INSEE_h
fit.rel_h[[2]] <- fit.m5.TG05_h

names(fit.rel_h) <- names(fit.rel_f) <- c("M5.A", "M5.TG05")

save(fit.rel_h, file="fit.rel_h.RData")


