

## ------------------------------------------------------------------------ ##
##  R scripts for " A New Parametric Approach to Smoothing and Forecasting  ##
##                 of Mortality "                                           ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts to compare the fits  -  Script n•05               ##
## ------------------------------------------------------------------------ ##
##  Description   Script to compare the fits - graphical // tests           ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 19/10/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...")

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("Misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ------------------------------------------------------------------------ ##
##           	             Load the fits and data                         ##
## ------------------------------------------------------------------------ ##

load("fit.all.h.RData")

load("D_h.RData")

load("ETR_h.RData")

## ------------------------------------------------------------------------ ##
##           	                Comparisons                                 ##
## ------------------------------------------------------------------------ ##

Age <- 30:90; Year <- 1996:2007

## ---------------- Graphical

isfa <- rgb(121,165,0, max=225)

col.vec <- c(1,2,2,isfa,isfa,4,4,"orange","orange")
pch.vec <- c(NA, 15, 0, 16, 1, 17, 2, 18, 5)
pch.vec <- c(rep(NA,9))
cex.vec <- c(1, 1.2, 1, 1.2, 1, 1.2, 1, 1.5, 1)
lwd.vec <- 2
lty.vec <- c(1, 1, 3, 1, 3, 1, 3, 1, 3)
leg.name <- c("M1", "M2.INSEE", "M2.TG05", "M3.INSEE", "M3.TG05", "M4.INSEE", "M4.TG05", "M5.INSEE", "M5.TG05")


## --- Comparison of the fit for a particular age: 

## - male population

x11();
comp.plot.age(fit.all.h, D_h/ETR_h, 30, c(90,148), col.vec, lty.vec, pch.vec, cex.vec, leg.name, as.character(Year),c(-8.1,-7.2))

x11();
comp.plot.age(fit.all.h, D_h/ETR_h, 40, c(90,148), col.vec, lty.vec, pch.vec, cex.vec, leg.name, as.character(Year),c(-7.1,-6.55))

x11();
comp.plot.age(fit.all.h, D_h/ETR_h, 50, c(90,148), col.vec, lty.vec,  pch.vec,cex.vec,leg.name, as.character(Year),c(-6.1,-5.65))

x11();
comp.plot.age(fit.all.h, D_h/ETR_h, 60, c(90,148), col.vec, lty.vec, pch.vec, cex.vec, leg.name, as.character(Year),c(-5.17,-4.7))

x11();
comp.plot.age(fit.all.h, D_h/ETR_h, 70, c(90,148), col.vec, lty.vec,  pch.vec, cex.vec, leg.name, as.character(Year),c(-4.2,-3.8))

x11();
comp.plot.age(fit.all.h, D_h/ETR_h, 80, c(90,148), col.vec, lty.vec, pch.vec, cex.vec, leg.name, as.character(Year),c(-3.06,-2.7))

## ---------------- Tests

## --- Standardized residuals

## - male population

test.sdresid <- stand.resid(fit.all.h, D_h/ETR_h, ETR_h, Age, as.character(Year), leg.name)

test.sdresid$s.r.2
test.sdresid$s.r.3

### --- Nber of runs & signs test:

## - male population

test.runs <- runs_test(fit.all.h, D_h/ETR_h, Age, as.character(Year), leg.name)

test.runs

test.sign <- sign_test(fit.all.h, D_h/ETR_h, Age, as.character(Year), leg.name)

test.sign

### --- Kolmogorov Smirnov test:

## - male population

ks.tests <- kstest(fit.all.h, D_h/ETR_h, Age, as.character(Year), leg.name)

ks.tests

### --- chi^2, R2, MAPE

## - male population

chi2R2MAPE <- chi2R2MAPE.test(fit.all.h, D_h, ETR_h, Age, as.character(Year), leg.name)

chi2R2MAPE

### --- Deviance

deviance_h <- vector(,9)
for (i in 1:9){
	deviance_h[i] <- 2*sum(D_h * log(D_h/(fit.all.h[[i]]*ETR_h))- (D_h - fit.all.h[[i]]*ETR_h), na.rm=T)}

deviance_h

load("mod.lk_h.RData")
deviance_h <- fitteddf <- vector(,5)
for (i in 1:5){
deviance_h[i] <-as.numeric(-2*mod.lk_h[[i]]$dp["lk"])
fitteddf[i] <-as.numeric(mod.lk_h[[i]]$dp["df1"]) }

deviance_h
mod.lk_h[[6]]$deviance
mod.lk_h[[7]]$deviance

fitteddf

sum(mod.lk_h[[6]]$hat)
sum(mod.lk_h[[7]]$hat)

SMR_h <- vector(,9)
names(SMR_h) <- c("M1", "M2.INSEE", "M2.TG05", "M3.INSEE", "M3.TG05", "M4.INSEE", "M4.TG05", "M5.INSEE", "M5.TG05")
	for (j in 1:9){
	mu_h <- fit.all.h[[j]]
	SMR_h[j] <- sum(D_h) /sum(ETR_h * mu_h) } 

SMR_h; (SMR_h - 1)*100




