

## ------------------------------------------------------------------------ ##
##  R scripts for " A New Parametric Approach to Smoothing and Forecasting  ##
##                 of Mortality "                                           ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts to forecast with relational  -  Script n•06.2     ##
## ------------------------------------------------------------------------ ##
##  Description   Script to forecast mortality tables with the relational   ##
##                models.                                                   ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 20/10/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...")

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("Misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ------------------------------------------------------------------------ ##
##           	                  Load Data                                 ##
## ------------------------------------------------------------------------ ##

load("TG05_m_h.RData")

load("INSEE_m_h.RData")

## ------------------------------------------------------------------------ ##
##           	                 Load models                                ##
## ------------------------------------------------------------------------ ##

## --- Relational

load("mod.rel_h.RData")

## ------------------------------------------------------------------------ ##
##           	                   Forecast                                 ##
## ------------------------------------------------------------------------ ##

pred.rel_h <- vector("list", 2)

Age.new <- 30:90; Year.new <- 1996:2035

## ------------------------------------------------------------------------ ##
##           	                   M5.INSEE                                 ##
## ------------------------------------------------------------------------ ##

Age.new <- 30:90; Year.new <- 1996:2035

## ---------------- Male population

par <- mod.rel_h[[1]]$rslt

x.new <- f.logit(INSEE_m_h[, as.character(Year.new)])

## --- Obtain predictions

pred <- exp(par[1]+par[2]*x.new)/(1+exp(par[1]+par[2]*x.new))

## --- Surface plot

x11();
surface_plot(pred, expression(widehat(mu)[h]), c(min(Age.new),max(Age.new), min(Year.new), max(Year.new)))

x11();
surface_plot(log(pred), expression(log(widehat(mu)[h])), c(min(Age.new),max(Age.new), min(Year.new), max(Year.new)))

## --- Store the forecast

pred.rel_h[[1]] <- pred

## ------------------------------------------------------------------------ ##
##           	                    M5.TG05                                 ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

par <- mod.rel_h[[2]]$rslt

x.new <- f.logit(TG05_m_h[, as.character(Year.new)])

## --- Obtain predictions

pred <- exp(par[1]+par[2]*x.new)/(1+exp(par[1]+par[2]*x.new))

## --- Surface plot

x11();
surface_plot(pred, expression(widehat(mu)[h]), c(min(Age.new),max(Age.new), min(Year.new), max(Year.new)))

x11();
surface_plot(log(pred), expression(log(widehat(mu)[h])), c(min(Age.new),max(Age.new), min(Year.new), max(Year.new)))

## --- Store the forecast

pred.rel_h[[2]] <- pred

## ------------------------------------------------------------------------ ##
##           	          Save results for relational                       ##
## ------------------------------------------------------------------------ ##

save(pred.rel_h,file="pred.rel_h.RData")


