

## ------------------------------------------------------------------------ ##
##  R scripts for " A New Parametric Approach to Smoothing and Forecasting  ##
##                 of Mortality "                                           ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts to close tables  -  Script n•08                   ##
## ------------------------------------------------------------------------ ##
##  Description   Script to close the tables with Model 6 Thatcher 1999     ##
##                Age: 30:120 - Year: 1996-2035                             ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 20/10/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...")

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("Misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ------------------------------------------------------------------------ ##
##           	             Load the fits and data                         ##
## ------------------------------------------------------------------------ ##

load("extrapolated_tables_h.RData")

## --- Compute adjusted number of death, with L_0 = 100000

Age <- 30:90; Year <- 1996:2035

L_0 <- 100000

adjusted_D_h <- adjusted_D_f <- vector("list", 9)
adjusted_ETR_h <- adjusted_ETR_f <- vector("list", 9)

for (i in 1:9){
	adjusted_D_h[[i]] <- adjusted_ETR_h[[i]] <- matrix(0,length(Age),length(Year))
	colnames(adjusted_D_h[[i]])  <- colnames(adjusted_ETR_h[[i]])  <- as.character(Year)
	adjusted_ETR_h[[i]][1,] <- L_0; 
	adjusted_D_h[[i]][1,] <- L_0 * extrapolated_tables_h[[i]][1,]
	for (j in 2:length(Age)){
		adjusted_ETR_h[[i]][j,] <- adjusted_ETR_h[[i]][j-1,] - adjusted_D_h[[i]][j-1,]
		adjusted_D_h[[i]][j,] <- adjusted_ETR_h[[i]][j,] * extrapolated_tables_h[[i]][j,] } }

x11();		
surface_plot(adjusted_D_h[[7]], expression(paste(widehat(D)[h])), c(min(Age),max(Age),min(Year),max(Year)))

## ---------------- Model 6 - Thatcher 1999

## --- Get mux from 80 to 130 years old
## --- Ideal would be to treat years separatly to obtain a smooth fit, but procedure would be to time consumming...

fit.cl = function(x, y, debut, fin, per.fin) {
	matfit <- matrix(,length(per.fin[1]:per.fin[2]),ncol(x))
	parfit <- matrix(,ncol(x),2)
	for(i in 1:ncol(x)){
		deb <- debut[i]-30; fi <- fin[i]-30
		yy <- y[deb:fi, as.character(1996:2035)]
		closure.fit.M6T <- ct.PM("mod6.thatcher99", "bin", c(.0001,.001) , deb:fi, x[deb:fi,], yy, per.fin-30)
		matfit[,i] <- closure.fit.M6T$closure.fit[,i] 
		parfit[i,] <- closure.fit.M6T$closure.par[i,]}
return(list(fit = matfit, par = parfit)) }

## ------------------------------------------------------------------------ ##
##           	                   mod.1                                    ##
## ------------------------------------------------------------------------ ##

Age.cl <- 30:120

## --- male population

cl.mod.1_h <- fit.cl(adjusted_D_h[[1]], adjusted_ETR_h[[1]], rep(72, length(Year)), rep(91, length(Year)), c(89,121))

cl.mod.1_h$par

x11();
cl.par.plot(cl.mod.1_h$par[,1], cl.mod.1_h$par[,2], c(expression(alpha), expression(beta)), "Est. regression parameters for Mod.1", "b", Year)

cl.mod.1.all_h <- rbind(extrapolated_tables_h[[1]][((min(Age.cl)+1):88)-min(Age.cl), as.character(Year)], cl.mod.1_h$fit)
colnames(cl.mod.1.all_h) <- as.character(Year)

x11();
surface_plot(cl.mod.1.all_h, expression(paste(widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

x11();
surface_plot(log(cl.mod.1.all_h), expression(paste("log ",widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.2.INSEE                              ##
## ------------------------------------------------------------------------ ##

## --- male population

cl.mod.2.INSEE_h <- fit.cl(adjusted_D_h[[2]], adjusted_ETR_h[[2]], rep(69, length(Year)), rep(85, length(Year)), c(82,121))

cl.mod.2.INSEE_h$par

x11();
cl.par.plot(cl.mod.2.INSEE_h$par[,1], cl.mod.2.INSEE_h$par[,2], c(expression(alpha), expression(beta)), "Est. regression parameters for Mod.2.INSEE", "b", Year)

cl.mod.2.INSEE.all_h <- rbind(extrapolated_tables_h[[2]][((min(Age.cl)+1):81)-min(Age.cl), as.character(Year)], cl.mod.2.INSEE_h$fit)
colnames(cl.mod.2.INSEE.all_h) <- as.character(Year)

x11();
surface_plot(cl.mod.2.INSEE.all_h, expression(paste(widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

x11();
surface_plot(log(cl.mod.2.INSEE.all_h), expression(paste("log ",widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.2.TG05                               ##
## ------------------------------------------------------------------------ ##

## --- male population

cl.mod.2.TG05_h <- fit.cl(adjusted_D_h[[3]], adjusted_ETR_h[[3]], rep(69, length(Year)), rep(85, length(Year)), c(82,121))

cl.mod.2.TG05_h$par

x11();
cl.par.plot(cl.mod.2.TG05_h$par[,1], cl.mod.2.TG05_h$par[,2], c(expression(alpha), expression(beta)), "Est. regression parameters for Mod.2.TG05", "b", Year)

cl.mod.2.TG05.all_h <- rbind(extrapolated_tables_h[[3]][((min(Age.cl)+1):81)-min(Age.cl), as.character(Year)], cl.mod.2.TG05_h$fit)
colnames(cl.mod.2.TG05.all_h) <- as.character(Year)

x11();
surface_plot(cl.mod.2.TG05.all_h, expression(paste(widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

x11();
surface_plot(log(cl.mod.2.TG05.all_h), expression(paste("log ",widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.3.INSEE                              ##
## ------------------------------------------------------------------------ ##

## --- male population

cl.mod.3.INSEE_h <- fit.cl(adjusted_D_h[[4]], adjusted_ETR_h[[4]], rep(64, length(Year)), rep(84, length(Year)), c(82,121))

cl.mod.3.INSEE_h$par

x11();
cl.par.plot(cl.mod.3.INSEE_h$par[,1], cl.mod.3.INSEE_h$par[,2], c(expression(alpha), expression(beta)), "Est. regression parameters for Mod.3.INSEE", "b", Year)

cl.mod.3.INSEE.all_h <- rbind(extrapolated_tables_h[[4]][((min(Age.cl)+1):81)-min(Age.cl), as.character(Year)], cl.mod.3.INSEE_h$fit)
colnames(cl.mod.3.INSEE.all_h) <- as.character(Year)

x11();
surface_plot(cl.mod.3.INSEE.all_h, expression(paste(widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

x11();
surface_plot(log(cl.mod.3.INSEE.all_h), expression(paste("log ",widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.3.TG05                               ##
## ------------------------------------------------------------------------ ##

## --- male population

cl.mod.3.TG05_h <- fit.cl(adjusted_D_h[[5]], adjusted_ETR_h[[5]], rep(67, length(Year)), rep(85, length(Year)), c(82,121))

cl.mod.3.TG05_h$par

x11();
cl.par.plot(cl.mod.3.TG05_h$par[,1], cl.mod.3.TG05_h$par[,2], c(expression(alpha), expression(beta)), "Est. regression parameters for Mod.3.TG05", "b", Year)

cl.mod.3.TG05.all_h <- rbind(extrapolated_tables_h[[5]][((min(Age.cl)+1):81)-min(Age.cl), as.character(Year)], cl.mod.3.TG05_h$fit)
colnames(cl.mod.3.TG05.all_h) <- as.character(Year)

x11();
surface_plot(cl.mod.3.TG05.all_h, expression(paste(widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

x11();
surface_plot(log(cl.mod.3.TG05.all_h), expression(paste("log ",widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.4.INSEE                              ##
## ------------------------------------------------------------------------ ##

## --- male population

cl.mod.4.INSEE_h <- fit.cl(adjusted_D_h[[6]], adjusted_ETR_h[[6]], rep(65, length(Year)), rep(84, length(Year)), c(82,121))

cl.mod.4.INSEE_h$par

x11();
cl.par.plot(cl.mod.4.INSEE_h$par[,1], cl.mod.4.INSEE_h$par[,2], c(expression(alpha), expression(beta)), "Est. regression parameters for Mod.4.INSEE", "b", Year)

cl.mod.4.INSEE.all_h <- rbind(extrapolated_tables_h[[6]][((min(Age.cl)+1):81)-min(Age.cl), as.character(Year)], cl.mod.4.INSEE_h$fit)

x11();
surface_plot(cl.mod.4.INSEE.all_h, expression(paste(widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))
colnames(cl.mod.4.INSEE.all_h) <- as.character(Year)

x11();
surface_plot(log(cl.mod.4.INSEE.all_h), expression(paste("log ",widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.4.TG05                               ##
## ------------------------------------------------------------------------ ##

## --- male population

cl.mod.4.TG05_h <- fit.cl(adjusted_D_h[[7]], adjusted_ETR_h[[7]], rep(65, length(Year)), rep(86, length(Year)), c(82,121))

cl.mod.4.TG05_h$par

x11();
cl.par.plot(cl.mod.4.TG05_h$par[,1], cl.mod.4.TG05_h$par[,2], c(expression(alpha), expression(beta)), "Est. regression parameters for Mod.4.TG05", "b", Year)

cl.mod.4.TG05.all_h <- rbind(extrapolated_tables_h[[7]][((min(Age.cl)+1):81)-min(Age.cl), as.character(Year)], cl.mod.4.TG05_h$fit)
colnames(cl.mod.4.TG05.all_h) <- as.character(Year)

x11();
surface_plot(cl.mod.4.TG05.all_h, expression(paste(widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

x11();
surface_plot(log(cl.mod.4.TG05.all_h), expression(paste("log ",widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.5.INSEE                              ##
## ------------------------------------------------------------------------ ##

## --- male population

cl.mod.5.INSEE_h <- fit.cl(adjusted_D_h[[8]], adjusted_ETR_h[[8]], rep(61,length(Year)), rep(85,length(Year)), c(82,121))

cl.mod.5.INSEE_h$par

x11();
cl.par.plot(cl.mod.5.INSEE_h$par[,1], cl.mod.5.INSEE_h$par[,2], c(expression(alpha), expression(beta)), "Est. regression parameters for Mod.5.INSEE", "b", Year)

cl.mod.5.INSEE.all_h <- rbind(extrapolated_tables_h[[8]][((min(Age.cl)+1):81)-min(Age.cl), as.character(Year)], cl.mod.5.INSEE_h$fit)
colnames(cl.mod.5.INSEE.all_h) <- as.character(Year)

x11();
surface_plot(cl.mod.5.INSEE.all_h, expression(paste(widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

x11();
surface_plot(log(cl.mod.5.INSEE.all_h), expression(paste("log ",widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.5.TG05                               ##
## ------------------------------------------------------------------------ ##

## --- male population

cl.mod.5.TG05_h <- fit.cl(adjusted_D_h[[9]], adjusted_ETR_h[[9]],rep(61,length(Year)), rep(85,length(Year)), c(82,121))

cl.mod.5.TG05_h$par

x11();
cl.par.plot(cl.mod.5.TG05_h$par[,1], cl.mod.5.TG05_h$par[,2], c(expression(alpha), expression(beta)), "Est. regression parameters for Mod.5.TG05", "b", Year)

cl.mod.5.TG05.all_h <- rbind(extrapolated_tables_h[[9]][((min(Age.cl)+1):81)-min(Age.cl), as.character(Year)], cl.mod.5.TG05_h$fit)
colnames(cl.mod.5.TG05.all_h) <- as.character(Year)

x11();
surface_plot(cl.mod.5.TG05.all_h, expression(paste(widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

x11();
surface_plot(log(cl.mod.5.TG05.all_h), expression(paste("log ",widehat(mu)[h])), c(min(Age.cl),max(Age.cl),min(Year),max(Year)))

## ------------------------------------------------------------------------ ##
##           	        Store the completed tables                          ##
## ------------------------------------------------------------------------ ##

extrapolated_tables_close.h <- vector("list",9)
extrapolated_tables_close.h[[1]] <- cl.mod.1.all_h
extrapolated_tables_close.h[[2]] <- cl.mod.2.INSEE.all_h
extrapolated_tables_close.h[[3]] <- cl.mod.2.TG05.all_h
extrapolated_tables_close.h[[4]] <- cl.mod.3.INSEE.all_h
extrapolated_tables_close.h[[5]] <- cl.mod.3.TG05.all_h
extrapolated_tables_close.h[[6]] <- cl.mod.4.INSEE.all_h
extrapolated_tables_close.h[[7]] <- cl.mod.4.TG05.all_h
extrapolated_tables_close.h[[8]] <- cl.mod.5.INSEE.all_h
extrapolated_tables_close.h[[9]] <- cl.mod.5.TG05.all_h

names(extrapolated_tables_close.h) <- c("M1", "M2.INSEE", "M2.TG05", "M3.INSEE", "M3.TG05", "M4.INSEE", "M4.TG05", "M5.INSEE", "M5.TG05")

save(extrapolated_tables_close.h, file="extrapolated_tables_close.h.RData")


### --- Store estimated regression parameters alpha, beta

est.reg.par.close.h <- vector("list",9)
est.reg.par.close.h[[1]] <- cl.mod.1_h$par
est.reg.par.close.h[[2]] <- cl.mod.2.INSEE_h$par
est.reg.par.close.h[[3]] <- cl.mod.2.TG05_h$par
est.reg.par.close.h[[4]] <- cl.mod.3.INSEE_h$par
est.reg.par.close.h[[5]] <- cl.mod.3.TG05_h$par
est.reg.par.close.h[[6]] <- cl.mod.4.INSEE_h$par
est.reg.par.close.h[[7]] <- cl.mod.4.TG05_h$par
est.reg.par.close.h[[8]] <- cl.mod.5.INSEE_h$par
est.reg.par.close.h[[9]] <- cl.mod.5.TG05_h$par

names(est.reg.par.close.h) <- c("M1", "M2.INSEE", "M2.TG05", "M3.INSEE", "M3.TG05", "M4.INSEE", "M4.TG05", "M5.INSEE", "M5.TG05")

save(est.reg.par.close.h, file="est.reg.par.close.h.RData")


