

## ------------------------------------------------------------------------ ##
##  R scripts for " A New Parametric Approach to Smoothing and Forecasting  ##
##                 of Mortality "                                           ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts for comparisons  -  Script n•09                   ##
## ------------------------------------------------------------------------ ##
##  Description   Script to compare the completed tables and estimated      ##
##                regression parameters from Model 6 Tahtcher 1999.         ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 20/10/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...")

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("Misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ------------------------------------------------------------------------ ##
##           	             Load the fits and data                         ##
## ------------------------------------------------------------------------ ##

load("D_h.RData")
load("ETR_h.RData")

load("extrapolated_tables_close.h.RData")

load("est.reg.par.close.h.RData")

## ------------------------------------------------------------------------ ##
##           	         Surface plots completed tables                     ##
## ------------------------------------------------------------------------ ##

Age <- 30:130; Year <- 1996:2035

for (i in 1:9){

x11();
print(surface_plot(extrapolated_tables_close.h[[i]], expression(paste(widehat(mu)[h])), c(min(Age),max(Age),min(Year),max(Year))))

x11();
print(surface_plot(log(extrapolated_tables_close.h[[i]]), expression(paste("log ",widehat(mu)[h])), c(min(Age),max(Age),min(Year),max(Year)))) }

## ------------------------------------------------------------------------ ##
##           	   Comparisons graphical - completed tables                 ##
## ------------------------------------------------------------------------ ##

isfa <- rgb(121,165,0, max=225)

col.vec <- c(1,2,2,isfa,isfa,4,4,"orange","orange")
pch.vec <- c(NA, 15, 0, 16, 1, 17, 2, 18, 5)
pch.vec <- c(rep(NA,7))
cex.vec <- c(1, 1.2, 1, 1.2, 1, 1.2, 1, 1.5, 1)
lwd.vec <- 2
lty.vec <- c(1, 1, 3, 1, 3, 1, 3, 1, 3)
leg.vec <- c("M1", "M2.INSEE", "M2.TG05", "M3.INSEE", "M3.TG05", "M4.INSEE", "M4.TG05", "M5.INSEE", "M5.TG05")

Year <- 1996:2007; pred.per <- 1996:2035

x11();
comp.plot.forecast.age(extrapolated_tables_close.h, D_h/ETR_h, 30, c(90,33), col.vec, lty.vec, pch.vec, cex.vec,leg.vec, as.character(pred.per),as.character(Year),c(-10.3,-7.25))

x11();
comp.plot.forecast.age(extrapolated_tables_close.h, D_h/ETR_h, 40, c(90,33), col.vec, lty.vec, pch.vec, cex.vec, leg.vec, as.character(pred.per),as.character(Year),c(-9,-6.4))

x11();
comp.plot.forecast.age(extrapolated_tables_close.h, D_h/ETR_h, 50, c(90,33), col.vec,lty.vec, pch.vec, cex.vec, leg.vec, as.character(pred.per),as.character(Year),c(-7.3,-5.6))

x11();
comp.plot.forecast.age(extrapolated_tables_close.h, D_h/ETR_h, 60, c(90,33), col.vec, lty.vec, pch.vec, cex.vec, leg.vec, as.character(pred.per),as.character(Year),c(-6,-4.7))

x11();
comp.plot.forecast.age(extrapolated_tables_close.h, D_h/ETR_h, 70, c(90,33), col.vec, lty.vec, pch.vec, cex.vec, leg.vec, as.character(pred.per),as.character(Year),c(-5.4,-3.6))

x11();
comp.plot.forecast.age(extrapolated_tables_close.h, D_h/ETR_h, 80, c(90,33), col.vec, lty.vec, pch.vec, cex.vec, leg.vec, as.character(pred.per),as.character(Year),c(-4.1,-2.7))

x11();
comp.plot.forecast.age(extrapolated_tables_close.h, D_h/ETR_h, 90, c(90,33), col.vec, lty.vec, pch.vec, cex.vec, leg.vec, as.character(pred.per),as.character(Year),c(-2.75,-1.5))

x11();
comp.plot.forecast.age.2(extrapolated_tables_close.h, D_h/ETR_h, 100, c(90,33), col.vec, lty.vec, pch.vec, cex.vec, leg.vec, as.character(pred.per),as.character(Year),c(-1.65,-0.73))

## ------------------------------------------------------------------------ ##
##          Comparisons graphical - estimated regression parameters         ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

## --- alpha

x11();
comp.ct.par.plot(est.reg.par.close.h, 1, 1996:2035, c(90,148),col.vec, lty.vec, pch.vec, cex.vec, leg.vec, c(5e-06, 2.4e-04), expression(paste("Comparison of estimated reg. parameters: ", alpha[t])))

## --- beta 

x11();
comp.ct.par.plot(est.reg.par.close.h, 2, 1996:2035, c(90,148), col.vec, lty.vec, pch.vec, cex.vec, leg.vec, c(.11, .145), expression(paste("Comparison of estimated reg. parameters: ", beta[t])))

## ------------------------------------------------------------------------ ##
##           	               Survival function                            ##
## ------------------------------------------------------------------------ ##

## --- 30 to 69

age_sf <- 30:69

S.x.30_69_h <- matrix(,length(age_sf),length(extrapolated_tables_close.h))

for (i in 1:length(extrapolated_tables_close.h)) {
		S.x.30_69_h[,i] <- survi.fct(1-exp(-extrapolated_tables_close.h[[i]][age_sf+1-min(Age), ]), 100, age_sf)}
		
## - Male population

x11()
surv.plot(S.x.30_69_h, "Survival function for ages 30 to 69", c(45,48), col.vec, lty.vec,pch.vec, cex.vec , leg.vec, age_sf)

## --- 40 to 79

age_sf <- 40:79

S.x.40_79_h <-  matrix(,length(age_sf),length(extrapolated_tables_close.h))

for (i in 1:length(extrapolated_tables_close.h)) {
		S.x.40_79_h[,i] <- survi.fct(1-exp(-extrapolated_tables_close.h[[i]][age_sf+1-min(Age), ]), 100, age_sf) }
		
## - Male population

x11()
surv.plot(S.x.40_79_h, "Survival function for ages 40 to 79", c(45,48), col.vec, lty.vec,pch.vec, cex.vec , leg.vec, age_sf)

## --- 50 to 89

age_sf <- 50:89

S.x.50_89_h <- matrix(,length(age_sf),length(extrapolated_tables_close.h))

for (i in 1:length(extrapolated_tables_close.h)) {
		S.x.50_89_h[,i] <- survi.fct(1-exp(-extrapolated_tables_close.h[[i]][age_sf+1-min(Age), ]), 100, age_sf) }

	
## - Male population

x11()
surv.plot(S.x.50_89_h, "Survival function for ages 50 to 89, males", c(45,48), col.vec, lty.vec,pch.vec, cex.vec , leg.vec, age_sf)

## --- 60 to 99

age_sf <- 60:99

S.x.60_99_h <-matrix(,length(age_sf),length(extrapolated_tables_close.h))

for (i in 1:length(extrapolated_tables_close.h)) {
		S.x.60_99_h[,i] <- survi.fct(1-exp(-extrapolated_tables_close.h[[i]][age_sf+1-min(Age), ]), 100, age_sf)}
		
## - Male population

x11()
surv.plot(S.x.60_99_h, "Survival function for ages 60 to 99, males", c(45,48), col.vec, lty.vec,pch.vec, cex.vec , leg.vec,age_sf)

## --- 70 to 109

age_sf <- 70:109

S.x.70_109_h <- matrix(,length(age_sf),length(extrapolated_tables_close.h))

for (i in 1:length(extrapolated_tables_close.h)) {
		S.x.70_109_h[,i] <- survi.fct(1-exp(-extrapolated_tables_close.h[[i]][age_sf+1-min(Age), ]), 100, age_sf) }
		
## - Male population

x11()		
surv.plot(S.x.70_109_h, "Survival function for ages 70 to 109, males", c(133,133),col.vec, lty.vec,pch.vec, cex.vec , leg.vec, age_sf)

## --- 80 to 119

age_sf <- 80:119

S.x.80_119_h <- matrix(,length(age_sf),length(extrapolated_tables_close.h))

for (i in 1:length(extrapolated_tables_close.h)) {
		S.x.80_119_h[,i] <- survi.fct(1-exp(-extrapolated_tables_close.h[[i]][age_sf+1-min(Age), ]), 100, age_sf) }
		
## - Male population

x11()
surv.plot(S.x.80_119_h, "Survival function for ages 80 to 119, males", c(133,133),col.vec, lty.vec,pch.vec, cex.vec , leg.vec,age_sf)

## ------------------------------------------------------------------------ ##
##           	             Partial life expectancy                        ##
## ------------------------------------------------------------------------ ##

Year <- 1996:2035

age.mat <- matrix(,6,length(Year))
age.mat[1,] <- 30:69
for (i in 2:6){ age.mat[i,] <- age.mat[i-1,] + 10 }

ple_h <- p.l.e(extrapolated_tables_close.h, age.mat,min(Age))
ple.name <- c("30:69", "40:79", "50:89", "60:99", "70:109", "80:119")

rownames(ple_h) <- ple.name
colnames(ple_h) <-  leg.vec

ple_h;

## ------------------------------------------------------------------------ ##
##          	             Median month at death                          ##
## ------------------------------------------------------------------------ ##

## - Assumption for non-integer ages 
## - Uniform distribution of death:

surv.f <- vector("list", 6)
surv.f[[1]] <- S.x.30_69_h
surv.f[[2]] <- S.x.40_79_h
surv.f[[3]] <- S.x.50_89_h
surv.f[[4]] <- S.x.60_99_h
surv.f[[5]] <- S.x.70_109_h
surv.f[[6]] <- S.x.80_119_h

Year <- 1996:2035

get.s.xx <- function(s,a) {
	x.xx <- (0:99)/100
	nrs <- nrow(s); ncs <- ncol(s)
	a.x <- a[1] : a[2]
	s.xx <- matrix(,100*(length(a.x)-1),nrs)
	a.xx <- vector(,100*(length(a.x)-1))
	for (i in 1:nrs) {
		l <- 0
		for (j in 1:(length(a.x)-1)) {
			for (k in 1:100) {
				s.xx[l+k,i] <- (1-x.xx[k])*s[i,j]+x.xx[k]*s[i,j+1] 
				a.xx[l+k] <- a.x[j]+x.xx[k] } 
			l <- l + 100 }
			}
	return(list(s.xx = s.xx, a.xx = a.xx)) }

S.xx <- vector("list",9)
for(i in 1:6){
		S.xx[[i]] <- vector("list",2)
		S.xx[[i]][[1]] <- S.xx[[i]][[2]] <- matrix(,(40-1)*100,9)
		for(j in 1:9){
			tt <- get.s.xx(t(surv.f[[i]][,j]),c(1,40))
			S.xx[[i]][[1]][,j] <- tt$a.xx
			S.xx[[i]][[2]][,j] <- tt$s.xx } }
	
med.month <- matrix(,6,9)
rownames(med.month) <- ple.name; colnames(med.month) <- leg.vec
for (i in 3:6) {
	for(j in 1:9){
	med.month[i,j] <- S.xx[[i]][[1]][max(which(S.xx[[i]][[2]][,j] >= 0.5))+1, j] } }

med.month

sd.Fx <- matrix(,6,9)
colnames(sd.Fx) <- leg.vec
rownames(sd.Fx) <- ple.name
for (i in 1:9) {
	for(j in 1:6){
		sd.Fx[j,i] <- sqrt(var(1-surv.f[[j]][,i])) } }
		
sd.Fx

H.Fx <- matrix(,6,9)
colnames(H.Fx) <- leg.vec
rownames(H.Fx) <- ple.name
for (i in 1:9) {
	for(j in 1:6){
		H.Fx[j,i] <- - mean(log(surv.f[[j]][,i]))/ sum(surv.f[[j]][,i]) } }
		
H.Fx

