

## ------------------------------------------------------------------------ ##
##  R scripts for " A New Parametric Approach to Smoothing and Forecasting  ##
##                 of Mortality "                                           ##
## ------------------------------------------------------------------------ ##
##  Script        Sources                                                   ##
## ------------------------------------------------------------------------ ##
##  Description   Sources for misc and graphical functions                  ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien - 24-07-12                                   ##
## ------------------------------------------------------------------------ ##


## ------------------------------------------------------------------------ ##
##           	               Graphic functions                            ##
## ------------------------------------------------------------------------ ##

## ---------------- Surface plot

surface_plot = function(xx, zexpr, axis){
wireframe(xx, zlab=list(zexpr, rot=90), ylab=list("Years"), xlab=list("Attained age"), aspect=c(2,1), drape = F, colorkey = F, screen = list(z = 20, x = -75), scales = list(arrows = F, y = list(at = c(seq(1,ncol(xx), by = 2)), labels = c(seq(axis[3],axis[4],by=2)), cex=.8, rot=45), x = list( at = c(seq(1,131, by = 10)), labels = c(seq(axis[1],axis[2], by = 10)),cex=.8))) }

## ---------------- Comparisons by attained age

comp.plot.age = function(x, obs, age ,k, col.vec, lty.vec, pch.vec, cex.vec, leg.vec, year.vec, lll) {
	
	expr <- paste("Comparison of the fits for the age: ", age)
	xx <- matrix(,length(year.vec),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- x[[i]][age+1-30, year.vec]
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(log(xx)) ~ rep(as.numeric(year.vec), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Year',
	ylab = expression(paste("log ", varphi[x][t])), type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(min(as.numeric(year.vec)),max(as.numeric(year.vec)),1), labels = seq(min(as.numeric(year.vec)),max(as.numeric(year.vec)),1)), y = list(alternating = 3, 
	tck = c(1,1), limits= lll)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = isfa)))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	panel.points(as.numeric(year.vec), log(obs)[age+1-30, year.vec], type="p", col=1, pch=16, lwd=2)
	for( i in 1:length(x)) { panel.points(as.numeric(year.vec), log(x[[i]][age+1-30, year.vec]), type="b", col=col.vec[i], lty = lty.vec[i] , pch=pch.vec[i], cex=cex.vec[i], lwd=2) }
	trellis.unfocus()
k1 <- draw.key(list(columns = 3, between.columns = 0.2, lines = list( type = "b", col = col.vec, lty = lty.vec, pch = pch.vec, lwd=2), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparisons parameters PCA 1

comp.plot.pca1 = function(x, tt ,k, col.vec, lty.vec, pch.vec, cex.vec, leg.vec, vect, lll, expr, yyl, bb, cc, nc) {
	xx <- matrix(,length(vect),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- as.numeric(x[[i]]$basis[,tt])
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(xx) ~ rep(vect, length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = "Age",
	ylab = yyl, type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(min(vect),max(vect),bb), labels = seq(min(vect),max(vect),bb)), y = list(alternating = 3, 
	tck = c(1,1), limits= lll)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = isfa)))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	for( i in 1:length(x)) { panel.points(vect, xx[,i], type="b", col=col.vec[i], lty = lty.vec[i] , pch=pch.vec[i], cex=cex.vec[i]*cc,lwd=2) }
	trellis.unfocus()
k1 <- draw.key(list(columns = nc, between.columns = 0.2, lines = list( type = "b", col = col.vec, lty = lty.vec, pch = pch.vec, lwd=2), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparisons parameters PCA 2

comp.plot.pca2 = function(x, tt ,k, col.vec, lty.vec, pch.vec, cex.vec, leg.vec, vect, lll, expr, yyl, bb, cc, nc) {
	xx <- matrix(,length(vect),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- as.numeric(x[[i]]$coeff[,tt+1])
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(xx) ~ rep(vect, length(x)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = "Year",
	ylab = yyl, type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(min(vect),max(vect),bb), labels = seq(min(vect),max(vect),bb)), y = list(alternating = 3, 
	tck = c(1,1), limits= lll)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = isfa)))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	for( i in 1:length(x)) { panel.points(vect, xx[,i], type="b", col=col.vec[i], lty = lty.vec[i] , pch=pch.vec[i], cex=cex.vec[i]*cc, lwd=2) }
	trellis.unfocus()
k1 <- draw.key(list(columns = nc, between.columns = 0.2, lines = list( type = "b", col = col.vec, lty = lty.vec, pch = pch.vec, lwd=2), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparisons projections of parameters

projection.plot = function(x, y, tt,year, period, k, col.vec, lty.vec, pch.vec, cex.vec, leg.vec, lll, expr, byy) {
	xx <- matrix(, length(year),length(x))
	yy <- matrix(, length(period),length(y))
	for (i in 1:length(x)) { xx[,i] <- as.numeric(x[[i]]$coeff[,tt+1]) }
	for (i in 1:length(x)) { yy[,i] <- as.numeric(y[[i]][[tt]]$mean) }
	zz <- rbind(xx,yy)
	nn <- length(c(zz)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(zz), grp1))
	zzz <- with(dat, xyplot(c(zz) ~ rep(1:nrow(zz), length(gr)) | as.factor(grp1), data = dat, layout = c(1,1), as.table = T, xlab = "Year", ylab = "", type= "n", scales = list(x = list(alternating = 1, tck = c(1,0), at = seq(1, nrow(zz), byy), labels = seq(min(year),max(period),byy), cex = 1), y = list(alternating = 3, tck = c(1,1), limits= lll)), par.settings = list(fontsize = list(text = 15, points = 7)), between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1), strip = strip.custom(which.given = 1, factor.levels = expr, bg = isfa)))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	for( i in 1:length(x)) { 
		panel.points(1:(nrow(zz)-nrow(yy)), zz[1:(nrow(zz)-nrow(yy)),i], type="b", col=col.vec[i],pch=pch.vec[i], lty = lty.vec[i], cex=cex.vec[i], lwd=2) 
		panel.points((nrow(zz)-nrow(yy)-1):nrow(zz), zz[(nrow(zz)-nrow(yy)-1):nrow(zz),i], type="b", col=col.vec[i], pch=pch.vec[i], lty = lty.vec[i], cex=cex.vec[i]*.9, lwd=2)  
#		panel.points((nrow(xx)):nrow(zz), c(zz[nrow(xx),i],y[[i]]$upper[,1]), type="b", col=1, pch=pch.vec[i], lty = 3, cex=cex.vec[i]*.5) 
#		panel.points((nrow(xx)):nrow(zz),c(zz[nrow(xx),i], y[[i]]$lower[,1]), type="b", col=1, pch=pch.vec[i], lty = 3, cex=cex.vec[i]*.5)
		}
	trellis.unfocus()
k1 <- draw.key(list(columns = 1, between.columns = 0.2, lines = list( type = "b", col = col.vec, pch=pch.vec, lty=lty.vec, lwd=2), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## --- Closure parameters plot

cl.par.plot = function(vec.obs1, vec.obs2, str.names, main.name, tt, lab) {
	nn <- length(c(vec.obs1, vec.obs2))/2
	gr <- c(1,2)
	grp1 <- as.factor(rep(gr, each = nn))
	dat <- as.data.frame(cbind(c(vec.obs1, vec.obs2), grp1))
	zzz <- with(dat, xyplot(c(vec.obs1, vec.obs2) ~ rep(1:nn, length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,2), as.table = T, xlab = "Years", main = main.name, 
	ylab = '', type= "b", lty=2, col = "black", scales = list(x = list(alternating = 1, 
	tck = c(1,0), at = seq(1,length(vec.obs1),5), 
	labels = seq(min(lab),max(lab),5) ), y = list(relation = "free",
	alternating = 3, tck = c(1,1))), par.settings = list(fontsize = list(text = 15,
	points = 7)), between=list(y = 0.6, x = 0.6), strip = strip.custom(which.given = 1,
	factor.levels = str.names, bg = gray(.95)), par.strip.text = list(cex = 1,
	lines = 1.1))); print(zzz) }
	
## ---------------- Comparisons forecasts by attained age

comp.plot.forecast.age = function(x, obs, age ,k, col.vec, lty.vec, pch.vec, cex.vec, leg.vec, year.vec, y1, lll) {
	
	expr <- paste("Comparison of the forecasts at age: ", age)
	xx <- matrix(,length(year.vec),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- x[[i]][age+1-30, year.vec]
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(log(xx)) ~ rep(as.numeric(year.vec), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Year',
	ylab = expression(paste("log ", varphi[x][t])), type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(min(as.numeric(year.vec)),max(as.numeric(year.vec)),5), labels = seq(min(as.numeric(year.vec)),max(as.numeric(year.vec)),5)), y = list(alternating = 3, 
	tck = c(1,1), limits= lll)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = isfa)))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	panel.points(as.numeric(y1), log(obs)[age+1-30, y1], type="p", col=1, pch=16)
	for( i in 1:length(x)) { panel.points(as.numeric(year.vec), log(x[[i]][age+1-30, year.vec]), type="b", col=col.vec[i], lty = lty.vec[i] , pch=pch.vec[i], cex=cex.vec[i], lwd=2) }
	trellis.unfocus()
k1 <- draw.key(list(columns = 3, between.columns = 0.2, lines = list( type = "b", col = col.vec, lty = lty.vec, pch = pch.vec, cex=cex.vec, lwd=2), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparisons forecasts by attained age 2

comp.plot.forecast.age.2 = function(x, obs, age ,k, col.vec, lty.vec, pch.vec, cex.vec, leg.vec, year.vec, y1, lll) {
	
	expr <- paste("Comparison of the forecasts at age: ", age)
	xx <- matrix(,length(year.vec),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- x[[i]][age+1-30, year.vec]
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(log(xx)) ~ rep(as.numeric(year.vec), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Year',
	ylab = expression(paste("log ", varphi[x][t])), type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(min(as.numeric(year.vec)),max(as.numeric(year.vec)),5), labels = seq(min(as.numeric(year.vec)),max(as.numeric(year.vec)),5)), y = list(alternating = 3, 
	tck = c(1,1), limits= lll)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = isfa)))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
#	panel.points(as.numeric(y1), log(obs)[age+1-30, y1], type="p", col=1, pch=16)
	for( i in 1:length(x)) { panel.points(as.numeric(year.vec), log(x[[i]][age+1-30, year.vec]), type="b", col=col.vec[i], lty = lty.vec[i] , pch=pch.vec[i], cex=cex.vec[i], lwd=2) }
	trellis.unfocus()
k1 <- draw.key(list(columns = 3, between.columns = 0.2, lines = list( type = "b", col = col.vec, lty = lty.vec, pch = pch.vec, cex=cex.vec, lwd=2), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparisons est. reg. parameters

comp.ct.par.plot = function(x, aa, year ,k, col.vec, lty.vec, pch.vec, cex.vec, leg.vec, lll, expr) {
	nnn <- c(expression(paste("",alpha)), expression(paste("",beta)))
	xx <- matrix(,length(year),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- x[[i]][, aa]
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(xx) ~ rep(1:length(year), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Year',
	ylab = "", type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(1,length(year),5), labels = seq(min(year),max(year),5)), y = list(alternating = 3, 
	tck = c(1,1), limits= lll)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = isfa)))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	for( i in 1:length(x)) { panel.points(1:length(year), x[[i]][,aa], type="b", col=col.vec[i], lty = lty.vec[i], pch=pch.vec[i], cex=cex.vec[i], lwd=2) }
	trellis.unfocus()
k1 <- draw.key(list(columns = 3, between.columns = 0.2, lines = list( type = "b", col = col.vec, lty = lty.vec, pch=pch.vec, cex=cex.vec, lwd=2), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparisons survival function

surv.plot = function(x,expr,k, col.vec,lty.vec, pch.vec, cex.vec , leg.vec, age.vec) {
	nn <- length(c(x)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(x), grp1))
	zzz <- with(dat, xyplot(c(x) ~ rep(1:nrow(x), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Age',
	ylab = "", type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(1,nrow(x),3), labels = seq(min(age.vec),max(age.vec),3)), y = list(alternating = 3, 
	tck = c(1,1))), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = isfa)))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	for( i in 1:ncol(x)) { panel.points(1:nrow(x), x[,i], type="b", col=col.vec[i], lty = lty.vec[i], cex=cex.vec[i]*.8, pch=pch.vec[i],lwd=2) }
	trellis.unfocus()
k1 <- draw.key(list(columns = 1, between.columns = 0.2, lines = list( type = "b", col = col.vec, lty = lty.vec, cex=cex.vec, pch=pch.vec,lwd=2), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ------------------------------------------------------------------------ ##
##           	          Local likelihood functions                        ##
## ------------------------------------------------------------------------ ##	
## ---------------- Compute criteria and fitted DF

get.crit.lk = function(z, y, x, hh, pp, k, fam, lk, ww) { 
	LCVMAT <- AICMAT <- V2MAT <- span.mat <- nber.p.mat <-matrix(,length(hh),length(pp))
	for ( h in hh ) {
		for( p in (pp+1) ) { 
			nber.p <- get.enp.target(h)
			nber.p.mat[h, p] <- nber.p
			span <- nber.p / (length(x)*length(y))
			span.mat[h, p] <- span
			LCVMAT[h, p] <- lcv(c(as.matrix(z)) ~ lp(c(rep(x,length(y))),c(rep(y,each= length(x))),deg = p-1, nn = span,  scale=1), ev=dat(), family = fam, link = lk, kern = k, weights = ww)[4]
			AICMAT[h, p] <- aic(c(as.matrix(z)) ~ lp(c(rep(x,length(y))),c(rep(y,each= length(x))),deg = p-1, nn = span,  scale=1), ev=dat(), family = fam, link = lk, kern = k, weights = ww)[4]
			V2MAT[h, p] <-  locfit(c(as.matrix(z)) ~ lp(c(rep(x,length(y))),c(rep(y,each= length(x))),deg = p-1, nn = span,  scale=1), ev=dat(), family = fam, link = lk, kern = k, weights = ww)$dp["df2"] } } 
	return(list(LCVMAT = LCVMAT, AICMAT = AICMAT, V2MAT = V2MAT, span.mat = span.mat, nber.p.mat = nber.p.mat )) }

## ---------------- Plot criterion

plot.crit = function(crit.name, crit, v2, lim.crit, lim.v2) { 
	figname <- paste(crit.name, "scores for various polynomial degrees")
	VECA <- c(crit); VECB <- c(v2)
	xyplot(VECA ~ VECB|as.factor(figname),xlab = "Fitted DF", ylab= crit.name, type="n", par.settings = list(fontsize = list(text = 15, points = 12)), par.strip.text = list(cex = 1, lines = 1.1), strip = strip.custom(bg = gray(.95)), scales = list(y = list(relation = "free", limits = list(lim.crit), at = list(c(seq(min(lim.crit),max(lim.crit),by = (max(lim.crit)-min(lim.crit))/10)))), x = list(relation = "free", limits = lim.v2, tck = c(1,1))), panel = function(...) {
		llines(VECA[1:20]~VECB[1:20],type = "p", pch = "0", col = 1)
		llines(VECA[21:40]~VECB[21:40],type = "p", pch = "1", col = 1)
		llines(VECA[41:60]~VECB[41:60],type = "p", pch = "2", col = 1)
		llines(VECA[61:80]~VECB[61:80],type = "p", pch = "3", col = 1) } ) }

## ---------------- Get the equivalent number of parameter
	
get.enp.target = function(h) {
	e.d.vec <- euc.dist(1:41, 1:41, 21, 21, c("mod.1", h))
	e.n.p.target <- length(which(e.d.vec <= h)==TRUE)
	return(e.n.p.target)
	}
	
## ---------------- This function returns the Euclidian distance if mod.1 is
## ---------------- specified. When mod.2 and mod.3 are specifed it scaled the
## ---------------- distance.

euc.dist = function(x1, x2, a1, a2, mod)
     {
      x1.vec <- rep(x1, times = length(x2))
      x2.vec <- rep(x2, each = length(x1))
      if (mod[1] == "mod.1") {
      VEC <- sqrt((a1 - x1.vec)^2 + ((a2 - x2.vec)^2)) }
      if (mod[1] == "mod.2") {
      h.1 <- ((as.numeric(mod[2]) * length(x1)) -1) / 2
	  h.2 <- ((as.numeric(mod[2]) * length(x2)) -1) / 2  
	  VEC <- sqrt((a1 - x1.vec)^2 + ((a2 - x2.vec) * (h.1 / h.2) )^2 ) }
	  if (mod[1] == "mod.3") {
      h.1 <- ((as.numeric(mod[2]) * length(x1)) -1) / 2
	  h.2 <- ((as.numeric(mod[3]) * length(x2)) -1) / 2
	  VEC <- sqrt((a1 - x1.vec)^2 + ((a2 - x2.vec) * (h.1 / h.2) )^2  ) }
	  if (mod[1] == "mod.4") {
      h.1 <- as.numeric(mod[2])
	  h.2 <- as.numeric(mod[3])
	  VEC <- sqrt((a1 - x1.vec)^2 + ((a2 - x2.vec) * (h.1 / h.2) )^2  ) }
	  if (mod[1] == "mod.5") {
      VEC <- sqrt((a1 - x1.vec)^2 + ((a2 - x2.vec)^2)) }
	 return(VEC)
     }

## ---------------- Define logit function

f.logit = function(x) { log(x/(1-x)) }
	
## ------------------------------------------------------------------------ ##
##  Description     Sets functions used for a relational model              ##
## ------------------------------------------------------------------------ ##
##                  Inspired by http://www.ressourcesactuarielles.net/r     ##
## ------------------------------------------------------------------------ ##

rel.mod = function(x, x_ref, w){
	logit_x <- f.logit(x)
	logit_x[logit_x == -Inf] <- 0
	logit_x_ref <- f.logit(x_ref)
	logit_x_ref[logit_x_ref == -Inf] <- 0
	dd = function(p){
		y <- (p[1] + p[2] * logit_x_ref)
		x_p <- exp(y) / (1 + exp(y))
		sum(abs(w * (x-x_p)))
		}
	rslt <- constrOptim(c(0,1), dd, ui = c(0, 1), ci = 0, control = list(maxit = 10^3), method = "Nelder-Mead")$par
	x_smooth <- exp(rslt[1]+rslt[2]*logit_x_ref)/(1+exp(rslt[1]+rslt[2]*logit_x_ref))
	return(list(x_smooth = x_smooth, rslt = rslt)) }

## ------------------------------------------------------------------------ ##
##           	                Tests functions                             ##
## ------------------------------------------------------------------------ ##	
## ---------------- Standardized residuals

stand.resid = function(x, y, z, age.vec, year.vec, leg.name) {
	s.r <- vector("list",length(x))
	names(s.r) <- leg.name
	s.r.2 <- s.r.3 <- matrix(,2,length(x))
	colnames(s.r.2) <- colnames(s.r.3) <- leg.name
	rownames(s.r.2) <- rownames(s.r.3) <- c("False", "True")
	for(i in 1:length(x)){
	xx <- x[[i]][age.vec + 1 - min(age.vec),year.vec]
	yy <-  y[age.vec + 1- min(age.vec), year.vec]
	zz <- z[age.vec + 1- min(age.vec), year.vec]
	s.r[[i]] <- (xx - yy) / sqrt(xx/zz)
	colnames(s.r[[i]]) <- year.vec
	s.r.2[,i] <- table(abs(s.r[[i]])>2)
	s.r.3[,i] <- table(abs(s.r[[i]])>3) } 
	return(list(s.r = s.r, s.r.2 = s.r.2, s.r.3 = s.r.3)) }

## ---------------- Runs test:

runs_test = function(x, y, age.vec, year.vec, leg.name){
rlt.all <- vector("list", length(x))
names(rlt.all ) <-  leg.name
rlt <- matrix(,5,length(x))
colnames(rlt) <- leg.name
rownames(rlt) <- c("Nber of runs", "Signs (-)", "Signs (+)", "Statistic", "p.value")
for (i in 1:length(x)) {
yy <- sign((y)[age.vec-min(age.vec)+1, year.vec ] -  x[[i]][age.vec + 1 - min(age.vec) , year.vec])
fac.SIGN <- factor(as.matrix(val.SIGN))
sign.neg <- sum(levels(fac.SIGN)[1] == fac.SIGN)
sign.pos <- sum(levels(fac.SIGN)[2] == fac.SIGN)
val.RUNS <- 1 + sum(as.numeric(fac.SIGN[-1] != fac.SIGN[-length(fac.SIGN)]))
mean.run <- 1 + 2*sign.neg*sign.pos / (sign.neg+sign.pos)
var.run <- 2*sign.neg*sign.pos * (2*sign.neg*sign.pos - sign.neg - sign.pos) / ((sign.neg+sign.pos)^2 * (sign.neg+sign.pos-1))
xi.RUNS <- (val.RUNS - mean.run) / sqrt(var.run)
p.val.RUNS <- 2*(1-pnorm(abs(xi.RUNS)))
rlt[1,i] <- val.RUNS
rlt[2,i] <- sign.neg; rlt[3,i] <- sign.pos; 
rlt[4,i] <- xi.RUNS; rlt[5,i] <- p.val.RUNS }
return(rlt = rlt) }

sign_test = function(x, y, age.vec, year.vec, leg.name){
rlt.all <- vector("list", length(x))
names(rlt.all ) <-  leg.name
rlt <- matrix(,5,length(x))
colnames(rlt) <- leg.name
rownames(rlt) <- c("Signs (-)", "Signs (+)", "Statistic", "p.value")
for (i in 1:length(x)) {
yy <- sign((y)[age.vec-min(age.vec)+1, year.vec ] -  x[[i]][age.vec + 1 - min(age.vec) , year.vec])
fac.SIGN <- factor(as.matrix(val.SIGN))
sign.neg <- sum(levels(fac.SIGN)[1] == fac.SIGN)
sign.pos <- sum(levels(fac.SIGN)[2] == fac.SIGN)
xi.SIGN <- (abs(sign.pos-sign.neg)-1) / sqrt(sign.pos+sign.neg)
p.val.SIGN <- 2*(1-pnorm(abs(xi.SIGN)))
rlt[1,i] <- sign.neg; rlt[2,i] <- sign.pos; 
rlt[3,i] <- xi.SIGN; rlt[4,i] <- p.val.SIGN }
return(rlt = rlt) }


## ---------------- Kolmogorov Smirnov test:

kstest = function(x, y, age.vec, year.vec, leg.name){
	rlt <- matrix(,2,length(x))
	colnames(rlt) <- leg.name
	rownames(rlt) <- c("Statistic", "p.value")
	for (i in 1:length(x)) {
		xx <- x[[i]][age.vec + 1 - min(age.vec), year.vec]
		yy <-  y[age.vec + 1 - min(age.vec), year.vec]
		kk <- ks.test(yy, xx)
		rlt[1,i] <- kk$statistic; rlt[2,i] <- kk$p.value 	}
		return(rlt)
}

## ---------------- chi^2, R2, MAPE

chi2R2MAPE.test = function(x, y, z, age.vec, year.vec, leg.name){
	rlt <- matrix(,3,length(x))
	colnames(rlt) <- leg.name
	rownames(rlt) <- c("Chi^2", "R2", "MAPE")
	for (i in 1:length(x)) {
		xx <- x[[i]][age.vec + 1 - min(age.vec), year.vec]
		yy <-  y[age.vec + 1- min(age.vec), year.vec]
		zz <- z[age.vec + 1- min(age.vec), year.vec]
		chi2 <- sum(((yy - zz * xx)^2) / (zz * xx * (1-xx)))
		R2 <- 1 - (sum((yy/zz - xx)^2) / sum((yy/zz - mean(yy/zz))^2))
		MAPE <- sum(abs((yy/zz - xx)/(yy/zz))[yy>0])/sum(yy>0) * 100
		rlt[1,i] <- chi2; rlt[2,i] <- R2; rlt[3,i] <- MAPE 	}
		return(rlt)
}

## ------------------------------------------------------------------------ ##
##               Closing the life table: parametric methods                 ##
## ------------------------------------------------------------------------ ##

## - Define the log-likelihood

bin.logl = function(mu,x) {
	logl <- sum(x[,2] * log(mu / (1 - mu))) + sum(x[,3] * log(1 - mu))
	return(logl) }

## - Define expression for the mean

f.expit = function(x) {
	VEC <- exp(x)/(1 + exp(x)); return(VEC) }

mu.mod6.thatcher99 = function(x,p) {
	mu <- f.expit(log(p[1]) + p[2] * x)
	return(mu) }

## - Define expressions to minizie

bin.lik.mod6.thatcher99 = function(x,p) {
	mu <- mu.mod6.thatcher99(x[,1],p)
	logl <- bin.logl(mu,x)
	return(-logl)
	}

## - Obtain parameters with "optim"

get.par = function(ini, x, type, family) {
	if (type == "mod6.thatcher99" & family == "bin" ) {
	par <- optim(ini ,bin.lik.mod6.thatcher99, x = x )$par }
	return(par) }

get.fit = function(type,x,p) {
	if(type == "mod6.thatcher99") { mu <- mu.mod6.thatcher99(x,p) }
	return(mu) }

get.xx = function(i, family, x1, x2, x3) {
	if (family == "bin") {xx <- cbind(x1,x2[,i],x3[,i]) }
	return(xx) }

ct.PM = function(type, family, ini, x1, x2, x3, c.r) {
	age.tbf <- c.r[1] : c.r[2]
	nn <- ncol(x2)
	closure.par <- matrix(,nn,length(ini))
	closure.fit <-  matrix(,length(age.tbf),nn)
	for (i in 1:nn) {
		xx <- get.xx(i, family, x1, x2, x3)
		closure.par[i,] <- get.par(ini, xx, type, family)
		for (j in 1:length(age.tbf)) {
		closure.fit[j,i] <- get.fit(type,age.tbf[j],closure.par[i,]) } }
	return(list(closure.par = closure.par, closure.fit = closure.fit)) }

## ------------------------------------------------------------------------ ##
##          	            Survival function                               ##
## ------------------------------------------------------------------------ ##

survi.fct = function(qx.p,l_0,x) {
	lx.p <- vector(,length(x))
	lx.p[1] <- l_0
	for (i in 1:length(x)) {
	lx.p[1+i] <- (1-qx.p[i,i])*lx.p[i] }
	S.x <- lx.p / l_0
	return(S.x = S.x[1:length(x)])
	}

## ------------------------------------------------------------------------ ##
##          	        Partial life expectancies                           ##
## ------------------------------------------------------------------------ ##

p.l.e = function(mu,age.mat,min.age){
	ple <- matrix(,nrow(age.mat),length(mu))
	for(a in 1:nrow(age.mat)){
		age.vec <- age.mat[a,]
	for (i in 1:length(mu)) {
		p1 <- cumprod(exp(-diag(mu[[i]][age.vec+1-min.age, ])))
		ple[a,i] <- sum(p1) } } 
		return(ple = ple) }






